package robust.pm.auth;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import robust.pm.property.CSProperty;
import robust.pm.util.CookieUtil;

import robust.pm.logger.CSLogger;



/**
 *  ó 
 *
 */
public class AuthManager {
	
    /** ⺻ Property */
    public static final String SYSTEM_PROPERTY = "base";

    /**   */
	public static final int IS_SESSION = 1;

    /** Ű  */
	public static final int IS_COOKIE = 2;
    
    
    /**  ⺻ Class */
    private static class AuthInfo {
    
        /** θ */
        public static String authDomainName;
        
        /**  ޼ҵ */
//        public static int authMethod = IS_COOKIE;
        public static int authMethod = IS_SESSION;
        
        /**  ӽð */ 
        //public static int authDuration = 4 * 60 * 60;	// ʴ
        public static int authDuration = 14 * 60 * 60;	// ʴ
        
        /**  ޼ҵ */
//        public static String authMethodName = "COOKIE";
        public static String authMethodName = "SESSION";
    
    }
    
    static {
        AuthInfo.authDomainName = CSProperty.getInstance().getValue("SERVICE_DOMAIN");
        AuthInfo.authDuration   = CSProperty.getInstance().getIntValue("AUTH_DURATION",-1);
        AuthInfo.authMethodName = CSProperty.getInstance().getValue("AUTH_METHOD");

        if ("SESSION".equals(AuthInfo.authMethodName)) {
            AuthInfo.authMethod = IS_SESSION;       // session 
        } else if ("COOKIE".equals(AuthInfo.authMethodName)) {
            AuthInfo.authMethod = IS_COOKIE;        // cookie 
        } else {
            AuthInfo.authMethod = IS_COOKIE;       // ⺻ cookie 
        }
    }

	/**
	 * Ʈ
     * ش Class new Ͽ  ϵ Ѵ.
	 * 
	 * @throws Exception
	 */
	private AuthManager() {

	}

	/**
	 * SSO α Ѵ.
	 * 
	 * @param req HttpServletRequest
	 * @param res HttpServletResponse
	 * @param conn Connection
	 * @param loginId String
     * 
	 * @throws Exception
	 * @return boolean
	 */
	public static boolean login(HttpServletRequest req, HttpServletResponse res,
			String loginId, String password) {
        
        // Է ̵  α  
        if(loginId == null) return false;

        // α ̵ ҹ 
		//loginId = loginId.toLowerCase();

		HashMap userInfo = null;
		
		try {
            LegacyUserInfoImpl LegacyUserInfo = new LegacyUserInfoImpl();
            userInfo = LegacyUserInfo.getUserInfo( loginId, password, true);
		} catch (Exception ex) {
			System.err.println("α  :"+ex.getMessage());
			ex.printStackTrace();
			return false;
		}
        
        if(userInfo == null) {
            return false;
        }
        
        String[][] sUserInfos = null;
        String[] sInfo = null;
        AuthEntity authEntity = new AuthEntity();
        
        sUserInfos = authEntity.getUserInfos();
        
        for(int i = 0; i < sUserInfos.length; i++) {
        	
            sInfo = sUserInfos[i];
            sInfo[AuthEntity.VALUE] = (String) userInfo.get(sInfo[AuthEntity.NAME]);
//System.out.println(sInfo[AuthEntity.NAME] + ":" + sInfo[AuthEntity.VALUE]);
            if(sInfo[AuthEntity.VALUE] == null) sInfo[AuthEntity.VALUE] = "";
            
            // Delimiter ϴ  Delimiter Ѵ.
            while(sInfo[AuthEntity.VALUE].indexOf(AuthEntity.DELIMITER) == 0) {
                sInfo[AuthEntity.VALUE] = sInfo[AuthEntity.VALUE].substring(AuthEntity.DELIMITER.length());
            }
        }
        
        switch(AuthInfo.authMethod) {
            case IS_COOKIE:
                setCookie(req, res, authEntity);
                return true;
            case IS_SESSION:
                setSession(req, res, authEntity);
                return true;
        }

		return false;
	}
    

	/**
	 * α׾ƿ Ѵ.
	 * 
	 * @param req HttpServletRequest
	 * @param res HttpServletResponse
	 * @param sabun  
     * 
	 * @throws Exception
	 * @return boolean α׾ƿ 
	 */
	public static boolean logOut(HttpServletRequest req, HttpServletResponse res,
			String loginId) throws Exception {
        
        AuthEntity authEntity = new AuthEntity();

		// α׾ƿ Ű 
        switch(AuthInfo.authMethod) {
            case IS_COOKIE:
                setCookie(req, res, authEntity);
                return true;
            case IS_SESSION:
                setSession(req, res, authEntity);
                return true;
        }

        return false;
	}
    
    /**
     * Ű Ǵ κ α  ȸѴ. 
     * 
     * @param req HttpServletRequest
     * 
     * @return α  
     */
    public static AuthEntity isLogin(HttpServletRequest req) {
        
        AuthEntity authEntity = null;
        
        // α׾ƿ Ű 
        switch(AuthInfo.authMethod) {
            case IS_COOKIE:
                authEntity = getCookie(req);
                break;
            case IS_SESSION:
                authEntity = getSession(req);
                break;
        }
        
        return authEntity;
    }

    /**
     *     Ѵ.
     *
     * @param req HttpServletResponse
     * @param res HttpServletResponse
     * @param authEntity   
     *
     */
    private static void setSession(HttpServletRequest req, HttpServletResponse res, AuthEntity authEntity) {
        
        String[][] sUserInfos = null;
        String[] sInfo = null;
        
        HttpSession ssn = req.getSession();
        
        sUserInfos = authEntity.getUserInfos();
        
        for(int i = 0; i < sUserInfos.length; i++) {
            
            sInfo = sUserInfos[i];
            
//            ssn.setAttribute(sInfo[AuthEntity.NAME], CryptoUtil.encrypt(sInfo[AuthEntity.VALUE]));
            ssn.setAttribute(sInfo[AuthEntity.NAME], sInfo[AuthEntity.VALUE]);
        }

        if(AuthInfo.authDuration > 0)
            ssn.setMaxInactiveInterval(AuthInfo.authDuration);
    }
    
    /**
     *    Ű Ѵ.
     *
     * @param req HttpServletResponse
     * @param res HttpServletResponse
     * @param authEntity   
     *
     */
    private static void setCookie(HttpServletRequest req, HttpServletResponse res, AuthEntity authEntity) {
        
        String[][] sUserInfos = null;
        String[] sInfo = null;
        
        sUserInfos = authEntity.getUserInfos();
        
        for(int i = 0; i < sUserInfos.length; i++) {
            
            sInfo = sUserInfos[i];
            
            Cookie c = null;
            
//            System.out.println(sInfo[AuthEntity.NAME] + " : " + CryptoUtil.encrypt(sInfo[AuthEntity.VALUE]));

//            c = new Cookie(sInfo[AuthEntity.NAME], CryptoUtil.encrypt(sInfo[AuthEntity.VALUE]));
              c = new Cookie(sInfo[AuthEntity.NAME], sInfo[AuthEntity.VALUE]);
              
            if(AuthInfo.authDuration > 0)
                c.setMaxAge(AuthInfo.authDuration);
            c.setPath("/");
            res.addCookie(c);
        }
    }
    
    /**
     * HttpRequest  Cookie мϿ  ȯѴ.
     *
     * @param req   Http Request
     *
     * @return 
     */
    private static AuthEntity getCookie(HttpServletRequest req) {
        
        AuthEntity              authEntity      = new AuthEntity();
        Cookie []               cookie          = req.getCookies();
        String                  sBuf            = null;
        String[][]              sUserInfos      = null;
        String[]                sInfo           = null;
        
        if(cookie == null)  return authEntity;
        
        sUserInfos = authEntity.getUserInfos();
        for(int i = 0; i < sUserInfos.length; i++) {
            sInfo = sUserInfos[i];
            sBuf = CookieUtil.getValue(cookie, sInfo[AuthEntity.NAME]);
            
            if(sBuf != null && !"".equals(sBuf))
            	sInfo[AuthEntity.VALUE] = sBuf;
//                sInfo[AuthEntity.VALUE] = CryptoUtil.decrypt(sBuf);
        }
        
        return authEntity;
    }
    
    /**
     *   Cookie о  ȯѴ.
     *
     * @param req   Http Request
     *
     * @return 
     */
    private static AuthEntity getSession(HttpServletRequest req) {
        
        AuthEntity              authEntity      = new AuthEntity();
        Cookie []               cookie          = req.getCookies();
        String                  sBuf            = null;
        String[][]              sUserInfos      = null;
        String[]                sInfo           = null;
        
        if(cookie == null)  return authEntity;
        
        HttpSession ssn = req.getSession(false);
        
        sUserInfos = authEntity.getUserInfos();
        for(int i = 0; i < sUserInfos.length; i++) {
            sInfo = sUserInfos[i];
            sBuf = (String)ssn.getAttribute(sInfo[AuthEntity.NAME]);
            
            if(sBuf != null && !"".equals(sBuf))
            	sInfo[AuthEntity.VALUE] = sBuf;
                //sInfo[AuthEntity.VALUE] = CryptoUtil.decrypt(sBuf);
        }
        
        return authEntity;
    }    
}
